@@ -9,7 +9,6 @@ import time
 import errno
 import logging
 import signal
-
 import begin
 
 import nbeam.version
@@ -20,6 +19,7 @@ DEFAULT_CONFIG_PATH = os.path.join(os.environ['HOME'], '.config', 'neutron-beam'
 DEFAULTS = {
   'config_dir': DEFAULT_CONFIG_PATH,
   'auto_reload': False,
+  'api_key': None,
   'port': 32828,
   'foreground': False,
   'view_timeout': 30,
@@ -32,9 +32,11 @@ DEFAULTS = {
   'encrypt': True,
 }
 
-def get_config (config_dir, **kwargs):
-  if not os.path.exists(config_dir):
-    os.makedirs(config_dir)
+def LOAD_DEFAULTS ():
+  config_dir = DEFAULTS['config_dir']
+  if '--config-dir' in sys.argv:
+    i = sys.argv.index('--config-dir')
+    config_dir = sys.argv[i + 1]
     
   json_config = os.path.join(config_dir, 'config.json')
   if os.path.exists(json_config):
@@ -42,31 +44,43 @@ def get_config (config_dir, **kwargs):
     config = json.loads(fh.read())
     fh.close()
     
-  else:
-    config = {}
-    
-  for key in DEFAULTS:
-    if key != 'config_dir':
-      if key not in config:
-        config[key] = DEFAULTS[key]
-        
-  for key in kwargs:
-    if kwargs[key] is not None:
-      config[key] = kwargs[key]
+    for key, value in config.items():
+      DEFAULTS[key] = value
       
+LOAD_DEFAULTS()
+
+def get_config (config_dir, **config):
+  if not os.path.exists(config_dir):
+    os.makedirs(config_dir)
+    
+  for key, value in DEFAULTS.items():
+    if key not in config:
+      config[key] = value
+      
+  json_config = os.path.join(config_dir, 'config.json')
+  
+  write = False
   if config['code_dir'] is None:
     config['code_dir'] = raw_input('Enter your code directory: ')
+    write = True
     
   if config['username'] is None:
     config['username'] = raw_input('Enter your Neutron Drive username: ')
+    write = True
+    
+  if config['api_key'] is None:
+    config['api_key'] = raw_input('Enter your Beam API Key (Leave blank for no encryption): ')
+    write = True
     
   if config['pid_file'] is None:
     config['pid_file'] = os.path.join(config_dir, 'nbeam.pid')
+    write = True
     
   if config['log_file'] is None:
     config['log_file'] = os.path.join(config_dir, 'nbeam.log')
+    write = True
     
-  if not os.path.exists(json_config):
+  if write or not os.path.exists(json_config):
     fh = open(json_config, 'w')
     fh.write(json.dumps(config, indent=2))
     fh.close()
@@ -93,6 +107,7 @@ def start (
     pid_file=DEFAULTS['pid_file'],
     log_file=DEFAULTS['log_file'],
     encrypt=DEFAULTS['encrypt'],
+    api_key=DEFAULTS['api_key'],
   ):
   "Start Neutron Beam"
   
@@ -109,6 +124,7 @@ def start (
     log_file=log_file,
     pid_file=pid_file,
     encrypt=encrypt,
+    api_key=api_key,
   )
   
   if not config['foreground']:
@@ -148,10 +164,10 @@ def get_pid (config):
     return pid
     
 @begin.subcommand
-def stop (config_dir=DEFAULT_CONFIG_PATH):
+def stop (config_dir=DEFAULT_CONFIG_PATH, pid_file=DEFAULTS['pid_file']):
   "Stop Neutron Beam"
   
-  config = get_config(config_dir)
+  config = get_config(config_dir, pid_file=pid_file)
   
   print('Stopping Neutron Beam ...')
   pid = get_pid(config)
@@ -178,6 +194,7 @@ def restart (
     pid_file=DEFAULTS['pid_file'],
     log_file=DEFAULTS['log_file'],
     encrypt=DEFAULTS['encrypt'],
+    api_key=DEFAULTS['api_key'],
   ):
   "Restart Neutron Beam"
   
@@ -195,6 +212,7 @@ def restart (
     log_file=log_file,
     pid_file=pid_file,
     encrypt=encrypt,
+    api_key=api_key,
   )
   
 @begin.subcommand
@@ -205,10 +223,10 @@ def config (config_dir=DEFAULT_CONFIG_PATH):
   print(json.dumps(config, indent=2))
   
 @begin.subcommand
-def running (config_dir=DEFAULT_CONFIG_PATH):
+def running (config_dir=DEFAULT_CONFIG_PATH, pid_file=DEFAULTS['pid_file']):
   "Print whether Neutron Beam is running"
   
-  config = get_config(config_dir)
+  config = get_config(config_dir, pid_file=pid_file)
   
   pid = get_pid(config)
   
